@@ -1,12 +1,11 @@
-"""System Bridge Connector: HTTP Client"""
+"""HTTP Client."""
 from __future__ import annotations
 
 import asyncio
-from typing import Any, Optional
+from typing import Any
 
 from aiohttp import ClientResponse, ClientSession
 from aiohttp.client_exceptions import ClientConnectorError, ServerDisconnectedError
-import async_timeout
 
 from .base import Base
 from .exceptions import AuthenticationException, ConnectionErrorException
@@ -23,26 +22,43 @@ class HTTPClient(Base):
         self,
         api_host: str,
         api_port: int,
-        api_key: str,
-        session: Optional[ClientSession] = None,
+        token: str,
+        session: ClientSession | None = None,
     ) -> None:
-        """Initialize the client."""
+        """Initialise the client."""
         super().__init__()
-        self._api_key = api_key
+        self._token = token
         self._base_url = f"http://{api_host}:{api_port}"
         self._session = session if session else ClientSession()
 
+    async def delete(
+        self,
+        path: str,
+        payload: Any | None,
+    ) -> Any:
+        """Make a DELETE request."""
+        response: ClientResponse = await self.request(
+            "DELETE",
+            f"{self._base_url}{path}",
+            headers={
+                **BASE_HEADERS,
+                "token": self._token,
+            },
+            json=payload,
+        )
+        return await response.json()
+
     async def get(
         self,
         path: str,
     ) -> Any:
-        """Make a GET request"""
+        """Make a GET request."""
         response: ClientResponse = await self.request(
             "GET",
             f"{self._base_url}{path}",
             headers={
                 **BASE_HEADERS,
-                "api-key": self._api_key,
+                "token": self._token,
             },
         )
         if "application/json" in response.headers.get("Content-Type", ""):
@@ -52,15 +68,15 @@ class HTTPClient(Base):
     async def post(
         self,
         path: str,
-        payload: Optional[Any],
+        payload: Any | None,
     ) -> Any:
-        """Make a POST request"""
+        """Make a POST request."""
         response: ClientResponse = await self.request(
             "POST",
             f"{self._base_url}{path}",
             headers={
                 **BASE_HEADERS,
-                "api-key": self._api_key,
+                "token": self._token,
             },
             json=payload,
         )
@@ -69,15 +85,15 @@ class HTTPClient(Base):
     async def put(
         self,
         path: str,
-        payload: Optional[Any],
+        payload: Any | None,
     ) -> Any:
-        """Make a PUT request"""
+        """Make a PUT request."""
         response: ClientResponse = await self.request(
             "PUT",
             f"{self._base_url}{path}",
             headers={
                 **BASE_HEADERS,
-                "api-key": self._api_key,
+                "token": self._token,
             },
             json=payload,
         )
@@ -91,7 +107,7 @@ class HTTPClient(Base):
     ) -> ClientResponse:
         """Make a request."""
         try:
-            async with async_timeout.timeout(20):
+            async with asyncio.timeout(20):
                 response: ClientResponse = await self._session.request(
                     method,
                     url,
