@@ -1,34 +1,34 @@
-"""System Bridge Connector: Version"""
+"""Version."""
 from __future__ import annotations
 
 from aiohttp import ClientSession
 from pkg_resources import parse_version
-from typing import Optional
 
-from systembridgeconnector.base import Base
-from systembridgeconnector.exceptions import ConnectionErrorException
-from systembridgeconnector.http_client import HTTPClient
-from systembridgeconnector.models.system import System
+from systembridgemodels.modules.system import System
 
-SUPPORTED_VERSION = "3.1.2"
+from .base import Base
+from .exceptions import ConnectionErrorException
+from .http_client import HTTPClient
+
+SUPPORTED_VERSION = "4.0.2"
 
 
 class Version(Base):
-    """Version"""
+    """Version."""
 
     def __init__(
         self,
         api_host: str,
         api_port: int,
-        api_key: str,
-        session: Optional[ClientSession] = None,
+        token: str,
+        session: ClientSession | None = None,
     ) -> None:
-        """Initialize the client."""
+        """Initialise the client."""
         super().__init__()
         self._http_client = HTTPClient(
             api_host,
             api_port,
-            api_key,
+            token,
             session,
         )
 
@@ -41,7 +41,7 @@ class Version(Base):
             return parse_version(version) >= parse_version(SUPPORTED_VERSION)
         return False
 
-    async def check_version_2(self) -> Optional[str]:
+    async def check_version_2(self) -> str | None:
         """Check if the system is running v2.x.x version."""
         try:
             information = await self._http_client.get("/information")
@@ -64,7 +64,7 @@ class Version(Base):
             raise exception
         return None
 
-    async def check_version_3(self) -> Optional[str]:
+    async def check_version_3(self) -> str | None:
         """Check if the system is running v3.x.x version."""
         try:
             response = await self._http_client.get("/api/data/system")
